import greenfoot.*;

/**
 * The enviroment for the simulation
 * 
 * @author TriMill
 * @version 1.0.2
 */
public class Earth extends World
{
    /**
     * The chance out of 10000 a piece of food will generate
     */
    private static final int FOOD_CHANCE = 30;
    /**
     * The chance out of 10000 a piece of super food will generate
     */
    private static final int SUPER_FOOD_CHANCE = 9;
    /**
     * The popup that shows statistics.
     */
    private InfoDialouge dialouge = new InfoDialouge();
    /**
     * A variable that is always equal to the current world, for use in
     * non-Actor classes that cant use the getWorld() method.
     */
    private static Earth world;
    
    /**
     * Creates the world.
     * The first for loop creates all of the food, the second for loop creates 
     * 10 creatures with random DNA, the two statements after that set up the 
     * statistics popup, and the last two set the act and paint order. The
     * final statement is used to set the static world varialbe, which can
     * be used instead of getWorld(), in extreme circumstances.
     */
    public Earth()
    {    
        // Create a new world with 600x400 cells with a cell size of 1x1 pixels.
        super(1100, 850, 1); 
        for(int i = 0; i < 10000; i++)
        {
            act();
        }
        for(int i = 0; i < 7; i++)
        {
            int x = Greenfoot.getRandomNumber(getWidth());
            int y = Greenfoot.getRandomNumber(getHeight());
            addObject(new Creature(), x, y);
        }
        dialouge.setSize(300, 300);
        dialouge.setLocationRelativeTo(null);
        
        setPaintOrder(Creature.class, SuperFood.class, Food.class, DeadCreature.class, Egg.class);
        setActOrder(Egg.class, DeadCreature.class, Creature.class);
        
        world = this;
    }
    
    /**
     * Causes stuff to go.
     * The first if statement has a chance of geenerating a piece of food, 
     * the second has a chance of generating a super food, the third checks
     * if the mouse has been clicked to make a new random creature, and the 
     * fourth removes creatures if the count goes above 45. Then, the world
     * is set and the popup box starts its version of the act method.
     */
    public void act() 
    {
        // Randomly generates food
        if(Greenfoot.getRandomNumber(10000) < FOOD_CHANCE)
        {
            int x = Greenfoot.getRandomNumber(getWidth());
            int y = Greenfoot.getRandomNumber(getHeight());
            addObject(new Food(), x, y);
        }
        // Randomly generates super food
        if(Greenfoot.getRandomNumber(10000) < SUPER_FOOD_CHANCE)
        {
            int x = Greenfoot.getRandomNumber(getWidth());
            int y = Greenfoot.getRandomNumber(getHeight());
            addObject(new SuperFood(), x, y);
        }
        // Checks for mouse clicks. If so, creates a new Creature
        if(Greenfoot.mouseClicked(null)) 
        {
            MouseInfo mouse = Greenfoot.getMouseInfo();
            Creature c = new Creature();
            addObject(c, mouse.getX(), mouse.getY());
        }
        
        if(countCreatures() > 45)
        {
            removeObject((Actor)getObjects(Creature.class).get(23));
        }
        world = this;
        dialouge.update();
    }
    
    // ------------------------------------------------------Get Methods------------------------------------------------------------ //
    
    /**
     * Gets the number of creatures in the world
     * 
     * @return The number of living Creatures in the world
     */
    public int countCreatures() { return getObjects(Creature.class).size(); }
    /**
     * Gets the number of food objects in the world
     * 
     * @return The sum of the number of Food objects and SuperFood objects in 
     * the world
     */
    public int countFood() { return getObjects(Food.class).size() + getObjects(Food.class).size(); }
    /**
     * Gets the number of eggs in the world
     * 
     * @return The number of Eggs in the world
     */
    public int countEggs() { return getObjects(Egg.class).size(); }
    
    /**
     * Returns the world, for use from non-Actor classes which dont have 
     * access to getWorld().
     * 
     * @return The current World
     */
    public static Earth getMe()
    {
        return world;
    }
}
